# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import logging
import sys

from reno import cache
from reno import config
from reno import create
from reno import defaults
from reno import linter
from reno import lister
from reno import report

_query_args = [
    (('--version',),
     dict(default=[],
          action='append',
          help='the version(s) to include, defaults to all')),
    (('--branch',),
     dict(default=config.Config.get_default('branch'),
          help='the branch to scan, defaults to the current')),
    (('--collapse-pre-releases',),
     dict(action='store_true',
          default=config.Config.get_default('collapse_pre_releases'),
          help='combine pre-releases with their final release')),
    (('--no-collapse-pre-releases',),
     dict(action='store_false',
          dest='collapse_pre_releases',
          help='show pre-releases separately')),
    (('--earliest-version',),
     dict(default=None,
          help='stop when this version is reached in the history')),
    (('--ignore-cache',),
     dict(default=False,
          action='store_true',
          help='if there is a cache file present, do not use it')),
    (('--stop-at-branch-base',),
     dict(action='store_true',
          default=True,
          dest='stop_at_branch_base',
          help='stop scanning when the branch meets master')),
    (('--no-stop-at-branch-base',),
     dict(action='store_false',
          dest='stop_at_branch_base',
          help='do not stop scanning when the branch meets master')),
]


def _build_query_arg_group(parser):
    group = parser.add_argument_group('query')
    for args, kwds in _query_args:
        group.add_argument(*args, **kwds)


def main(argv=sys.argv[1:]):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-v', '--verbose',
        dest='verbosity',
        default=logging.INFO,
        help='produce more output',
        action='store_const',
        const=logging.DEBUG,
    )
    parser.add_argument(
        '-q', '--quiet',
        dest='verbosity',
        action='store_const',
        const=logging.WARN,
        help='produce less output',
    )
    parser.add_argument(
        '--rel-notes-dir', '-d',
        dest='relnotesdir',
        default=defaults.RELEASE_NOTES_SUBDIR,
        help='location of release notes YAML files',
    )
    subparsers = parser.add_subparsers(
        title='commands',
    )

    do_new = subparsers.add_parser(
        'new',
        help='create a new note',
    )
    do_new.add_argument(
        '--edit',
        action='store_true',
        help='Edit note after its creation (require EDITOR env variable)',
    )
    do_new.add_argument(
        '--from-template',
        help='Template to get the release note from.',
    )
    do_new.add_argument(
        'slug',
        help='descriptive title of note (keep it short)',
    )
    do_new.add_argument(
        'reporoot',
        default='.',
        nargs='?',
        help='root of the git repository',
    )
    do_new.set_defaults(func=create.create_cmd)

    do_list = subparsers.add_parser(
        'list',
        help='list notes files based on query arguments',
    )
    _build_query_arg_group(do_list)
    do_list.add_argument(
        'reporoot',
        default='.',
        nargs='?',
        help='root of the git repository',
    )
    do_list.set_defaults(func=lister.list_cmd)

    do_report = subparsers.add_parser(
        'report',
        help='generate release notes report',
    )
    do_report.add_argument(
        'reporoot',
        default='.',
        nargs='?',
        help='root of the git repository',
    )
    do_report.add_argument(
        '--output', '-o',
        default=None,
        help='output filename, defaults to stdout',
    )
    do_report.add_argument(
        '--no-show-source',
        dest='show_source',
        default=True,
        action='store_false',
        help='do not show the source for notes',
    )
    do_report.add_argument(
        '--title',
        default='Release Notes',
        help='set the main title of the generated report',
    )
    _build_query_arg_group(do_report)
    do_report.set_defaults(func=report.report_cmd)

    do_cache = subparsers.add_parser(
        'cache',
        help='generate release notes cache',
    )
    do_cache.add_argument(
        'reporoot',
        default='.',
        nargs='?',
        help='root of the git repository',
    )
    do_cache.add_argument(
        '--output', '-o',
        default=None,
        help=('output filename, '
              'defaults to the cache file within the notesdir, '
              'use "-" for stdout'),
    )
    _build_query_arg_group(do_cache)
    do_cache.set_defaults(func=cache.cache_cmd)

    do_linter = subparsers.add_parser(
        'lint',
        help='check some common mistakes',
    )
    do_linter.add_argument(
        'reporoot',
        default='.',
        nargs='?',
        help='root of the git repository',
    )
    do_linter.set_defaults(func=linter.lint_cmd)

    args = parser.parse_args(argv)

    logging.basicConfig(
        level=args.verbosity,
        format='%(message)s',
    )

    conf = config.Config(args.reporoot, args.relnotesdir)
    conf.override_from_parsed_args(args)

    return args.func(args, conf)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import sys

import yaml

from reno import loader
from reno import scanner


def build_cache_db(conf, versions_to_include):
    s = scanner.Scanner(conf)
    notes = s.get_notes_by_version()

    # Default to including all versions returned by the scanner.
    if not versions_to_include:
        versions_to_include = list(notes.keys())

    # Build a cache data structure including the file contents as well
    # as the basic data returned by the scanner.
    file_contents = {}
    for version in versions_to_include:
        for filename, sha in notes[version]:
            body = s.get_file_at_commit(filename, sha)
            # We want to save the contents of the file, which is YAML,
            # inside another YAML file. That looks terribly ugly with
            # all of the escapes needed to format it properly as
            # embedded YAML, so parse the input and convert it to a
            # data structure that can be serialized cleanly.
            y = yaml.safe_load(body)
            file_contents[filename] = y

    cache = {
        'notes': [
            {'version': k, 'files': v}
            for k, v in notes.items()
        ],
        'file-contents': file_contents,
    }
    return cache


def write_cache_db(conf, versions_to_include,
                   outfilename=None):
    """Create a cache database file for the release notes data.

    Build the cache database from scanning the project history and
    write it to a file within the project.

    By default, the data is written to the same file the scanner will
    try to read when it cannot look at the git history. If outfilename
    is given and is '-' the data is written to stdout
    instead. Otherwise, if outfilename is given, the data overwrites
    the named file.

    Return the name of the file created, if any.

    """
    if outfilename == '-':
        stream = sys.stdout
        close_stream = False
    elif outfilename:
        stream = open(outfilename, 'w')
        close_stream = True
    else:
        outfilename = loader.get_cache_filename(conf)
        if not os.path.exists(os.path.dirname(outfilename)):
            os.makedirs(os.path.dirname(outfilename))
        stream = open(outfilename, 'w')
        close_stream = True
    try:
        cache = build_cache_db(
            conf,
            versions_to_include=versions_to_include,
        )
        yaml.safe_dump(
            cache,
            stream,
            allow_unicode=True,
            explicit_start=True,
            encoding='utf-8',
        )
    finally:
        if close_stream:
            stream.close()
    return outfilename


def cache_cmd(args, conf):
    "Generates a release notes cache"
    write_cache_db(
        conf=conf,
        versions_to_include=args.version,
        outfilename=args.output,
    )
    return



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from reno import config
from reno import formatter
from reno import loader
from reno.tests import base


class TestFormatterBase(base.TestCase):

    scanner_output = {
        '0.0.0': [('note1', 'shaA')],
        '1.0.0': [('note2', 'shaB'), ('note3', 'shaC')],
    }

    versions = ['0.0.0', '1.0.0']

    def _get_note_body(self, reporoot, filename, sha):
        return self.note_bodies.get(filename, '')

    def setUp(self):
        super(TestFormatterBase, self).setUp()

        def _load(ldr):
            ldr._scanner_output = self.scanner_output
            ldr._cache = {
                'file-contents': self.note_bodies
            }

        self.c = config.Config('reporoot')

        with mock.patch('reno.loader.Loader._load_data', _load):
            self.ldr = loader.Loader(
                self.c,
                ignore_cache=False,
            )


class TestFormatter(TestFormatterBase):

    note_bodies = {
        'note1': {
            'prelude': 'This is the prelude.',
        },
        'note2': {
            'issues': [
                'This is the first issue.',
                'This is the second issue.',
            ],
        },
        'note3': {
            'features': [
                'We added a feature!',
            ],
            'upgrade': None,
        },
    }

    def test_with_title(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title='This is the title',
        )
        self.assertIn('This is the title', result)

    def test_versions(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title='This is the title',
        )
        self.assertIn('0.0.0\n=====', result)
        self.assertIn('1.0.0\n=====', result)

    def test_without_title(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title=None,
        )
        self.assertNotIn('This is the title', result)

    def test_default_section_order(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title=None,
        )
        prelude_pos = result.index('This is the prelude.')
        issues_pos = result.index('This is the first issue.')
        features_pos = result.index('We added a feature!')
        expected = [prelude_pos, features_pos, issues_pos]
        actual = list(sorted([prelude_pos, features_pos, issues_pos]))
        self.assertEqual(expected, actual)


class TestFormatterCustomSections(TestFormatterBase):
    note_bodies = {
        'note1': {
            'prelude': 'This is the prelude.',
        },
        'note2': {
            'features': [
                'This is the first feature.',
            ],
            'api': [
                'This is the API change for the first feature.',
            ],
        },
        'note3': {
            'api': [
                'This is the API change for the second feature.',
            ],
            'features': [
                'This is the second feature.',
            ],
        },
    }

    def setUp(self):
        super(TestFormatterCustomSections, self).setUp()
        self.c.override(sections=[
            ['api', 'API Changes'],
            ['features', 'New Features'],
        ])

    def test_custom_section_order(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title=None,
        )
        prelude_pos = result.index('This is the prelude.')
        api_pos = result.index('API Changes')
        features_pos = result.index('New Features')
        expected = [prelude_pos, api_pos, features_pos]
        actual = list(sorted([prelude_pos, features_pos, api_pos]))
        self.assertEqual(expected, actual)
        self.assertIn('.. _relnotes_1.0.0_API Changes:', result)


class TestFormatterCustomUnreleaseTitle(TestFormatterBase):

    note_bodies = {
        'note1': {
            'prelude': 'This is the prelude.',
        },
    }

    scanner_output = {
        '0.1.0-1': [('note1', 'shaA')],
    }

    versions = ['0.1.0-1']

    def test_with_title(self):
        self.c.override(unreleased_version_title='Not Released')
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title='This is the title',
        )
        self.assertIn('Not Released', result)
        self.assertNotIn('0.1.0-1', result)
        self.assertIn('.. _This is the title_Not Released:', result)

    def test_without_title(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title='This is the title',
        )
        self.assertIn('0.1.0-1', result)
        self.assertIn('.. _This is the title_0.1.0-1:', result)


class TestFormatterAnchors(TestFormatterBase):

    note_bodies = {
        'note1': {
            'prelude': 'This is the prelude.',
        },
        'note2': {
            'issues': [
                'This is the first issue.',
                'This is the second issue.',
            ],
        },
        'note3': {
            'features': [
                'We added a feature!',
            ],
            'upgrade': None,
        },
    }

    def test_with_title(self):
        self.c.override(unreleased_version_title='Not Released')
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title='This is the title',
        )
        self.assertIn('.. _This is the title_0.0.0:', result)
        self.assertIn('.. _This is the title_0.0.0_Prelude:', result)
        self.assertIn('.. _This is the title_1.0.0:', result)
        self.assertIn('.. _This is the title_1.0.0_Known Issues:', result)

    def test_without_title(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
        )
        self.assertIn('.. _relnotes_0.0.0:', result)
        self.assertIn('.. _relnotes_0.0.0_Prelude:', result)
        self.assertIn('.. _relnotes_1.0.0:', result)
        self.assertIn('.. _relnotes_1.0.0_Known Issues:', result)

    def test_with_branch_and_title(self):
        self.c.override(unreleased_version_title='Not Released')
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            title='This is the title',
            branch='stable/queens',
        )
        self.assertIn('.. _This is the title_0.0.0_stable_queens:', result)
        self.assertIn('.. _This is the title_0.0.0_stable_queens_Prelude:',
                      result)
        self.assertIn('.. _This is the title_1.0.0_stable_queens:', result)
        self.assertIn(
            '.. _This is the title_1.0.0_stable_queens_Known Issues:',
            result)

    def test_with_branch(self):
        result = formatter.format_report(
            loader=self.ldr,
            config=self.c,
            versions_to_include=self.versions,
            branch='stable/queens',
        )
        self.assertIn('.. _relnotes_0.0.0_stable_queens:', result)
        self.assertIn('.. _relnotes_0.0.0_stable_queens_Prelude:', result)
        self.assertIn('.. _relnotes_1.0.0_stable_queens:', result)
        self.assertIn('.. _relnotes_1.0.0_stable_queens_Known Issues:', result)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import textwrap

from reno._exts import show_reno_config
from reno import config
from reno.tests import base


class TestMultiLineString(base.TestCase):

    def test_no_indent(self):
        input = textwrap.dedent("""\
        The notes subdirectory within the relnotesdir where the
        notes live.
        """)
        expected = '\n'.join([
            'The notes subdirectory within the relnotesdir where the',
            'notes live.',
        ])
        actual = '\n'.join(show_reno_config._multi_line_string(input))
        self.assertEqual(expected, actual)

    def test_with_indent(self):
        input = textwrap.dedent("""\
        The notes subdirectory within the relnotesdir where the
        notes live.
        """)
        expected = '\n'.join([
            '  The notes subdirectory within the relnotesdir where the',
            '  notes live.',
        ])
        actual = '\n'.join(show_reno_config._multi_line_string(input, '  '))
        self.assertEqual(expected, actual)

    def test_first_line_blank(self):
        input = textwrap.dedent("""
        The notes subdirectory within the relnotesdir where the
        notes live.
        """)
        expected = '\n'.join([
            '  The notes subdirectory within the relnotesdir where the',
            '  notes live.',
        ])
        actual = '\n'.join(show_reno_config._multi_line_string(input, '  '))
        self.assertEqual(expected, actual)


class TestFormatOptionHelp(base.TestCase):

    def test_simple_default(self):
        opt = config.Opt(
            'notesdir', 'path/to/notes',
            textwrap.dedent("""\
            The notes subdirectory within the relnotesdir where the
            notes live.
            """),
        )
        actual = '\n'.join(show_reno_config._format_option_help([opt]))
        expected = textwrap.dedent("""\
        ``notesdir``
          The notes subdirectory within the relnotesdir where the
          notes live.

          Defaults to ``'path/to/notes'``
        """)
        self.assertEqual(expected, actual)

    def test_bool_default(self):
        opt = config.Opt(
            'collapse_pre_releases', True,
            textwrap.dedent("""\
            Should pre-release versions be merged into the final release
            of the same number (1.0.0.0a1 notes appear under 1.0.0).
            """),
        )
        actual = '\n'.join(show_reno_config._format_option_help([opt]))
        expected = textwrap.dedent("""\
        ``collapse_pre_releases``
          Should pre-release versions be merged into the final release
          of the same number (1.0.0.0a1 notes appear under 1.0.0).

          Defaults to ``True``
        """)
        self.assertEqual(expected, actual)

    def test_multiline_default(self):
        opt = config.Opt(
            'release_tag_re',
            textwrap.dedent('''\
            ((?:[\d.ab]|rc)+)  # digits, a, b, and rc cover regular and
                               # pre-releases
            '''),
            textwrap.dedent("""\
            The regex pattern used to match the repo tags representing a
            valid release version. The pattern is compiled with the
            verbose and unicode flags enabled.
            """),
        )
        actual = '\n'.join(show_reno_config._format_option_help([opt]))
        expected = textwrap.dedent("""\
        ``release_tag_re``
          The regex pattern used to match the repo tags representing a
          valid release version. The pattern is compiled with the
          verbose and unicode flags enabled.

          Defaults to

          ::

            ((?:[\d.ab]|rc)+)  # digits, a, b, and rc cover regular and
                               # pre-releases
        """)
        self.assertEqual(expected, actual)



# Copyright 2017, Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Custom distutils command.

For more information, refer to the distutils and setuptools source:

- https://github.com/python/cpython/blob/3.6/Lib/distutils/cmd.py
- https://github.com/pypa/setuptools/blob/v36.0.0/setuptools/command/sdist.py
"""

from distutils import cmd
from distutils import errors
from distutils import log

import six

from reno import cache
from reno import config
from reno import defaults
from reno import formatter
from reno import loader

COMMAND_NAME = 'build_reno'  # duplicates what's found in setup.cfg


def load_config(distribution):
    """Utility method to parse distutils/setuptools configuration.

    This is for use by other libraries to extract the command configuration.

    :param distribution: A :class:`distutils.dist.Distribution` object
    :returns: A tuple of a :class:`reno.config.Config` object, the output path
        of the human-readable release notes file, and the output file of the
        reno cache file
    """
    option_dict = distribution.get_option_dict(COMMAND_NAME)

    if option_dict.get('repo_root') is not None:
        repo_root = option_dict.get('repo_root')[1]
    else:
        repo_root = defaults.REPO_ROOT

    if option_dict.get('rel_notes_dir') is not None:
        rel_notes_dir = option_dict.get('rel_notes_dir')[1]
    else:
        rel_notes_dir = defaults.RELEASE_NOTES_SUBDIR

    if option_dict.get('output_file') is not None:
        output_file = option_dict.get('output_file')[1]
    else:
        output_file = defaults.RELEASE_NOTES_FILENAME

    conf = config.Config(repo_root, rel_notes_dir)
    cache_file = loader.get_cache_filename(conf)

    return (conf, output_file, cache_file)


class BuildReno(cmd.Command):
    """Distutils command to build reno release notes.

    The release note build can be triggered from distutils, and some
    configuration can be included in ``setup.py`` or ``setup.cfg`` instead of
    being specified from the command-line.
    """
    description = 'Build reno release notes'
    user_options = [
        ('repo-root=', None, 'the root directory of the Git repository; '
         'defaults to "."'),
        ('rel-notes-dir=', None, 'the parent directory; defaults to '
         '"releasenotes"'),
        ('output-file=', None, 'the filename of the release notes file'),
    ]

    def initialize_options(self):
        self.repo_root = None
        self.rel_notes_dir = None
        self.output_file = None

    def finalize_options(self):
        if self.repo_root is None:
            self.repo_root = defaults.REPO_ROOT

        if self.rel_notes_dir is None:
            self.rel_notes_dir = defaults.RELEASE_NOTES_SUBDIR

        if self.output_file is None:
            self.output_file = defaults.RELEASE_NOTES_FILENAME

    # Overriding distutils' Command._ensure_stringlike which doesn't support
    # unicode, causing finalize_options to fail if invoked again. Workaround
    # for http://bugs.python.org/issue19570
    def _ensure_stringlike(self, option, what, default=None):
        # type: (unicode, unicode, Any) -> Any
        val = getattr(self, option)
        if val is None:
            setattr(self, option, default)
            return default
        elif not isinstance(val, six.string_types):
            raise errors.DistutilsOptionError("'%s' must be a %s (got `%s`)"
                                              % (option, what, val))
        return val

    def run(self):
        conf = config.Config(self.repo_root, self.rel_notes_dir)

        # Generate the cache using the configuration options found
        # in the release notes directory and the default output
        # filename.
        cache_filename = cache.write_cache_db(
            conf=conf,
            versions_to_include=[],  # include all versions
            outfilename=None,  # generate the default name
        )
        log.info('wrote cache file to %s', cache_filename)

        ldr = loader.Loader(conf)
        text = formatter.format_report(
            ldr,
            conf,
            ldr.versions,
            title=self.distribution.metadata.name,
        )
        with open(self.output_file, 'w') as f:
            f.write(text)
        log.info('wrote release notes to %s', self.output_file)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from docutils import nodes
from docutils.parsers import rst
from docutils.statemachine import ViewList

from sphinx.util.nodes import nested_parse_with_titles

from reno import config
import six


def _multi_line_string(s, indent=''):
    output_lines = s.splitlines()
    if not output_lines[0].strip():
        output_lines = output_lines[1:]
    for l in output_lines:
        yield indent + l


def _format_option_help(options):
    "Produce RST lines for the configuration options."
    for opt in options:
        yield '``{}``'.format(opt.name)
        for l in _multi_line_string(opt.help, '  '):
            yield l
        yield ''
        if isinstance(opt.default, six.string_types) and '\n' in opt.default:
            # Multi-line string
            yield '  Defaults to'
            yield ''
            yield '  ::'
            yield ''
            for l in _multi_line_string(opt.default, '    '):
                yield l
        else:
            yield '  Defaults to ``{!r}``'.format(opt.default)
        yield ''


class ShowConfigDirective(rst.Directive):

    option_spec = {}

    has_content = True

    def run(self):
        env = self.state.document.settings.env
        app = env.app

        result = ViewList()
        source_name = '<' + __name__ + '>'
        for line in _format_option_help(config._OPTIONS):
            app.info(line)
            result.append(line, source_name)

        node = nodes.section()
        node.document = self.state.document
        nested_parse_with_titles(self.state, result, node)

        return node.children


def setup(app):
    app.add_directive('show-reno-config', ShowConfigDirective)
